import java.io.*;
import java.net.*;
import java.util.StringTokenizer;

class TimeRegistrar
{
	private boolean rd = false;
	private String timeServer = null;


	public TimeRegistrar() {
		timeServer = "localhost";
	}

	public TimeRegistrar(String host) {
		timeServer = host;
	}

	public boolean registered() {
		return rd;
	}

	public void register() {
		try {
			System.out.println("connecting to "+timeServer);


			Socket sock;

			sock = new Socket(timeServer,9991);

			if (sock.isConnected())
				rd = true;
			// send the port number to the TimeServer
			// for now send 9990, because that is where time
			// server will contact you before i fix itp
			sock.close();

		} catch( Exception e ) {
			System.out.println("Problems registering with the time server ("+timeServer+")."); 
		}
    }
}


class ClientWorker implements Runnable {
	private Socket client;
	private Display display;
  
	ClientWorker(Socket client, Display display) {
		this.client = client;
		this.display = display;
	}

	public void run(){
		String line;
		DataInputStream input = null;
		try {
			input = new DataInputStream(client.getInputStream());
		} catch (IOException e) {
			System.out.println("read failed");
		}

		try{
			line = input.readUTF();

// 			System.out.println("Got "+line);

			StringTokenizer st = new StringTokenizer(line, ":");
			if (st.countTokens() < 3)
				display.append("** Invalid string ("+line+") received from "+client.getInetAddress().getHostName());

			String cmd = st.nextToken().toLowerCase();
			String arg1 = st.nextToken().toLowerCase();
			String arg2 = st.nextToken();

			String msg = "";
			if (cmd.equals("tick")) {
				display.setPeriod(arg2);
				String arg3 = st.nextToken();
				String arg4 = st.nextToken();

				if (arg4.equals("1")) {
					display.setRound("PAUSED");
					display.setClock("PAUSED");
				} else {
					display.setRound(arg1);
					display.setClock(arg3);
				}
			} else if (cmd.equals("status")) {
				display.setStatus(arg1, arg2);
			} else if (cmd.equals("dialog")) {
				display.append(arg1, arg2);
			} else if (cmd.equals("alarm")) {
				display.setAlarm(arg1, arg2);
			} else if (cmd.equals("treasure")) {
				display.setTreasure(arg1, arg2);
			} else
				display.append(line);
		} catch (IOException e) {
			System.out.println("Read failed");
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
}


public class Server {
	Display display = null;
	ServerSocket server = null;

	public Server(Display display) {
		this.display = display;

		TimeRegistrar tr = new TimeRegistrar();
	}
	
	public void listenSocket() {
		try {
			server = new ServerSocket(9990 );
		} catch (IOException e) {
			System.out.println("Could not listen on port 9990");
			System.exit(-1);
		}

		while (true) {
			ClientWorker w;
			try {
				w = new ClientWorker(server.accept(), display);
				Thread t = new Thread(w);
				t.start();
			} catch (IOException e) {
				System.out.println("Accept failed: 4123");
				System.exit(-1);
			}
		}
	}

	protected void finalize() {
		try {
			server.close();
		} catch (IOException e) {
			System.out.println("Could not close socket");
			System.exit(-1);
		}
	}


	public void run(){
		ServerSocket server;
		Socket connection;
		DataOutputStream output;
		DataInputStream input;
		int counter = 1;
		
		try{
			server = new ServerSocket(4123, 100);
			
			while (true){
				connection = server.accept();
				display.append("Connection "+counter+" received from " + connection.getInetAddress().getHostName());
				
				input = new DataInputStream(connection.getInputStream());
				output = new DataOutputStream(connection.getOutputStream());
				display.append("\nGot I/O streams\n");
				
				display.append ("Sending message \"Connection Successful\"\n");
				output.writeUTF("Connection Successful");
				display.append("Client message: "+input.readUTF());
				
				display.append("\nTransmission complete.  closing socket...\n");
				connection.close();
				++counter;
			}
		}
		catch (IOException e){
			e.printStackTrace();
		}
	}
}
